@@ -11,6 +11,7 @@ CREATE FUNCTION logdel_maker(
     , p_pulldata boolean DEFAULT true
     , p_pk_name text[] DEFAULT NULL
     , p_pk_type text[] DEFAULT NULL
+    , p_jobmon boolean DEFAULT NULL
     , p_debug boolean DEFAULT false) 
 RETURNS void
     LANGUAGE plpgsql
@@ -27,10 +28,13 @@ v_dblink_name               text;
 v_dblink_schema             text;
 v_dest_schema_name          text;
 v_dest_table_name           text;
-v_exists                    int := 0;
+v_exists                    text;
 v_field                     text;
 v_insert_refresh_config     text;
 v_jobmon                    boolean;
+v_job_id                    bigint;
+v_job_name               text;
+v_jobmon_schema             text;
 v_key_type                  text;
 v_link_exists               boolean;
 v_old_search_path           text;
@@ -52,6 +56,7 @@ v_source_queue_trigger      text;
 v_src_schema_name           text;
 v_src_table_name            text;
 v_src_table_template        text;
+v_step_id                   bigint;
 v_table_exists              boolean;
 v_trigger_func              text;
 v_types                     text[];
@@ -61,8 +66,9 @@ BEGIN
 v_dblink_name := @extschema@.check_name_length('mimeo_logdel_maker_'||p_src_table);
 
 SELECT nspname INTO v_dblink_schema FROM pg_namespace n, pg_extension e WHERE e.extname = 'dblink' AND e.extnamespace = n.oid;
+SELECT nspname INTO v_jobmon_schema FROM pg_namespace n, pg_extension e WHERE e.extname = 'pg_jobmon' AND e.extnamespace = n.oid;
 SELECT current_setting('search_path') INTO v_old_search_path;
-EXECUTE 'SELECT set_config(''search_path'',''@extschema@,'||v_dblink_schema||',public'',''false'')';
+EXECUTE 'SELECT set_config(''search_path'',''@extschema@,'||COALESCE(v_jobmon_schema||',', '')||v_dblink_schema||',public'',''false'')';
 
 IF (p_pk_name IS NULL AND p_pk_type IS NOT NULL) OR (p_pk_name IS NOT NULL AND p_pk_type IS NULL) THEN
     RAISE EXCEPTION 'Cannot manually set primary/unique key field(s) without defining type(s) or vice versa';
@@ -83,6 +89,22 @@ ELSE
     RAISE EXCEPTION 'Source (and destination) table must be schema qualified';
 END IF;
 
+IF p_jobmon IS TRUE AND v_jobmon_schema IS NULL THEN
+    RAISE EXCEPTION 'p_jobmon parameter set to TRUE, but unable to determine if pg_jobmon extension is installed';
+ELSIF (p_jobmon IS TRUE OR p_jobmon IS NULL) AND v_jobmon_schema IS NOT NULL THEN
+    v_jobmon := true;
+ELSE
+    v_jobmon := false;
+END IF;
+
+v_job_name := 'LogDel Maker: '||p_src_table;
+
+IF v_jobmon THEN
+    v_job_id := add_job(v_job_name);
+    PERFORM gdb(p_debug,'Job ID: '||v_job_id::text);
+    v_step_id := add_step(v_job_id,'Connecting to remote source');
+END IF;
+
 PERFORM dblink_connect(v_dblink_name, @extschema@.auth(p_dblink_id));
 
 SELECT schemaname ||'_'|| tablename, schemaname, tablename
@@ -105,6 +127,11 @@ IF v_src_table_template IS NULL THEN
     RAISE EXCEPTION 'Source table given (%) does not exist in configured source database', p_src_table;
 END IF;
 
+IF v_jobmon THEN
+    PERFORM update_step(v_step_id, 'OK','Done');
+    v_step_id := add_step(v_job_id,'Creating triggers & queue table on source');
+END IF;
+
 v_source_queue_table :=  check_name_length(v_src_table_template, '_q');
 v_source_queue_function := check_name_length(v_src_table_template, '_mimeo_queue');
 v_source_queue_trigger := check_name_length(v_src_table_template, '_mimeo_trig');
@@ -158,14 +185,6 @@ WHILE v_source_queue_exists IS NOT NULL LOOP -- loop until a tablename that does
     v_source_queue_trigger := check_name_length(v_src_table_template, '_mimeo_trig'||to_char(v_source_queue_counter, 'FM00'));
 END LOOP;
 
-SELECT 
-    CASE 
-        WHEN count(nspname) > 0 THEN true
-        ELSE false
-    END AS jobmon_schema
-INTO v_jobmon 
-FROM pg_namespace n, pg_extension e WHERE e.extname = 'pg_jobmon' AND e.extnamespace = n.oid;
-
 -- Unlike dml, config table insertion has to go first so that remote queue table creation step can have full column list
 v_insert_refresh_config := 'INSERT INTO @extschema@.refresh_config_logdel(
         source_table
@@ -200,6 +219,11 @@ INTO v_dest_schema_name, v_dest_table_name
 FROM pg_catalog.pg_tables 
 WHERE schemaname||'.'||tablename = p_dest_table;
 
+IF v_jobmon THEN
+    PERFORM update_step(v_step_id, 'OK','Done');
+    v_step_id := add_step(v_job_id,'Creating triggers & queue table on source');
+END IF;
+
 v_remote_q_table := format('CREATE TABLE %I.%I ('||array_to_string(v_cols_n_types, ',')||', mimeo_source_deleted timestamptz, processed boolean)', '@extschema@', v_source_queue_table);
 -- Indexes on queue table created below so the variable can be reused
 
@@ -261,58 +285,139 @@ PERFORM dblink_exec(v_dblink_name, v_trigger_func);
 PERFORM gdb(p_debug, 'v_create_trig: '||v_create_trig); 
 PERFORM dblink_exec(v_dblink_name, v_create_trig);
 
+IF v_jobmon THEN
+    PERFORM update_step(v_step_id, 'OK','Done');
+END IF;
+
 IF p_pulldata AND v_table_exists = false THEN
+    IF v_jobmon THEN
+        v_step_id := add_step(v_job_id,'Pulling data from source. Check for new job entry under REFRESH LOGDEL for current status if this step has not finished');
+    END IF;
     RAISE NOTICE 'Pulling all data from source...';
     EXECUTE 'SELECT refresh_logdel('||quote_literal(p_dest_table)||', p_repull := true, p_debug := '||p_debug||')';
+    IF v_jobmon THEN
+        PERFORM update_step(v_step_id, 'OK','Done');
+    END IF;
 END IF;
 
 IF p_index AND v_table_exists = false THEN
+    IF v_jobmon THEN
+        v_step_id := add_step(v_job_id,'Creating indexes on destination table');
+    END IF;
     PERFORM create_index(p_dest_table, v_src_schema_name, v_src_table_name, NULL, p_debug);
     -- Create index on special column for logdel
     EXECUTE format('CREATE INDEX %I ON %I.%I (mimeo_source_deleted)', check_name_length(v_dest_table_name, '_mimeo_source_deleted'), v_dest_schema_name, v_dest_table_name);
+    IF v_jobmon THEN
+        PERFORM update_step(v_step_id, 'OK','Done');
+    END IF;
 ELSIF v_table_exists = false THEN
     -- Ensure destination indexes that are needed for efficient replication are created even if p_index is set false
+    IF v_jobmon THEN
+        v_step_id := add_step(v_job_id,'Creating minimal required indexes on destination table');
+    END IF;
     RAISE NOTICE 'Adding primary/unique key to table...';
     IF v_key_type = 'primary' THEN
         EXECUTE format('ALTER TABLE %I.%I ADD PRIMARY KEY', v_dest_schema_name, v_dest_table_name) ||'("'||array_to_string(v_pk_name, '","')||'")';
     ELSE
         EXECUTE format('CREATE UNIQUE INDEX ON %I.%I', v_dest_schema_name, v_dest_table_name) ||'("'||array_to_string(v_pk_name, '","')||'")';
     END IF;
+    IF v_jobmon THEN
+        PERFORM update_step(v_step_id, 'OK','Done');
+    END IF;
 END IF;
 
 IF v_table_exists THEN
+    IF v_jobmon THEN
+        v_step_id := add_step(v_job_id, format('Destination table %s.%s already exists. No data or indexes were pulled from source', v_dest_schema_name, v_dest_table_name));
+        PERFORM update_step(v_step_id, 'OK','Done');
+    END IF;
     RAISE NOTICE 'Destination table % already exists. No data or indexes were pulled from source: %. Recommend making index on special column mimeo_source_deleted if it doesn''t have one', p_dest_table, p_src_table;
 END IF;
 
+IF v_jobmon THEN
+    v_step_id := add_step(v_job_id, 'Analyzing destination table');
+END IF;
+
 RAISE NOTICE 'Analyzing destination table...';
 EXECUTE format('ANALYZE %I.%I', v_dest_schema_name, v_dest_table_name);
 
+IF v_jobmon THEN
+    PERFORM update_step(v_step_id, 'OK','Done');
+END IF;
+
 PERFORM dblink_disconnect(v_dblink_name);
 
+IF v_jobmon THEN
+    PERFORM close_job(v_job_id);
+END IF;
+
 EXECUTE 'SELECT set_config(''search_path'','''||v_old_search_path||''',''false'')';
 
 RAISE NOTICE 'Done';
 RETURN;
 
 EXCEPTION
+    WHEN QUERY_CANCELED THEN
+        SELECT nspname INTO v_dblink_schema FROM pg_namespace n, pg_extension e WHERE e.extname = 'dblink' AND e.extnamespace = n.oid;
+        EXECUTE format('SELECT %I.dblink_get_connections() @> ARRAY[%L]', v_dblink_schema, v_dblink_name) INTO v_link_exists;
+        IF v_link_exists THEN
+            EXECUTE format('SELECT %I.dblink_disconnect(%L)', v_dblink_schema, v_dblink_name);
+        END IF;
+        RAISE EXCEPTION '%', SQLERRM;
     WHEN OTHERS THEN
         -- Only cleanup remote objects if replication doesn't exist at all for source table
         SELECT nspname INTO v_dblink_schema FROM pg_namespace n, pg_extension e WHERE e.extname = 'dblink' AND e.extnamespace = n.oid;
-        EXECUTE 'SELECT count(*) FROM @extschema@.refresh_config_logdel WHERE source_table = '||quote_literal(p_src_table) INTO v_exists;
+        SELECT nspname INTO v_jobmon_schema FROM pg_namespace n, pg_extension e WHERE e.extname = 'pg_jobmon' AND e.extnamespace = n.oid;
+        SELECT jobmon, dest_table INTO v_jobmon, v_exists FROM @extschema@.refresh_config_logdel WHERE dest_table = p_src_table;
+        v_jobmon := COALESCE(p_jobmon, v_jobmon);
+        IF v_jobmon_schema IS NULL THEN
+            v_jobmon := false;
+        END IF;
         EXECUTE format('SELECT %I.dblink_get_connections() @> ARRAY[%L]', v_dblink_schema, v_dblink_name) INTO v_link_exists;
+        IF v_jobmon THEN
+            IF v_job_id IS NULL THEN
+                EXECUTE format('SELECT %I.add_job(%L)', v_jobmon_schema, 'LogDel Maker: '||p_src_table) INTO v_job_id;
+                EXECUTE format('SELECT %I.add_step(%L, %L)', v_jobmon_schema, v_job_id, 'EXCEPTION before job logging started') INTO v_step_id;
+            END IF;
+            IF v_step_id IS NULL THEN
+                EXECUTE format('SELECT %I.add_step(%L, %L)', v_jobmon_schema, v_job_id, 'EXCEPTION before first step logged') INTO v_step_id;
+            END IF;
+            EXECUTE format('SELECT %I.update_step(%L, %L, %L)', v_jobmon_schema, v_step_id, 'CRITICAL', 'ERROR: '||COALESCE(SQLERRM,'unknown'));
+        END IF;
         IF v_link_exists THEN
-            IF v_exists = 0 THEN
-                EXECUTE format('%I.dblink_exec(%L, %L)', v_dblink_schema, v_dblink_name, format('DROP TRIGGER IF EXISTS %I ON %I.%I', v_source_queue_trigger, v_src_schema_name, v_src_table_name));
-                EXECUTE format('%I.dblink_exec(%L, %L)', v_dblink_schema, v_dblink_name, format('DROP TABLE IF EXISTS %I.%I', '@extschema@', v_source_queue_table));
-                EXECUTE format('%I.dblink_exec(%L, %L)', v_dblink_schema, v_dblink_name, format('DROP FUNCTION IF EXISTS %I.%I()', '@extschema@', v_source_queue_function));
+            IF v_exists IS NULL THEN
+                IF v_jobmon THEN
+                    EXECUTE format('SELECT %I.add_step(%L, %L)', v_jobmon_schema, v_job_id, 'Removing trigger & queue tables on source') INTO v_step_id;
+                END IF;
+                EXECUTE format('SELECT %I.dblink_exec(%L, %L)', v_dblink_schema, v_dblink_name, format('DROP TRIGGER IF EXISTS %I ON %I.%I', v_source_queue_trigger, v_src_schema_name, v_src_table_name));
+                EXECUTE format('SELECT %I.dblink_exec(%L, %L)', v_dblink_schema, v_dblink_name, format('DROP TABLE IF EXISTS %I.%I', '@extschema@', v_source_queue_table));
+                EXECUTE format('SELECT %I.dblink_exec(%L, %L)', v_dblink_schema, v_dblink_name, format('DROP FUNCTION IF EXISTS %I.%I()', '@extschema@', v_source_queue_function));
+                IF v_jobmon THEN
+                    EXECUTE format('SELECT %I.update_step(%L, %L, %L)', v_jobmon_schema, v_step_id, 'OK', 'Done');
+                END IF;
             END IF;
             EXECUTE format('SELECT %I.dblink_disconnect(%L)', v_dblink_schema, v_dblink_name);
         END IF;
-        IF v_exists = 0 AND v_link_exists THEN
+        IF v_exists IS NULL AND v_link_exists THEN
+            IF v_jobmon THEN
+                EXECUTE format('SELECT %I.add_step(%L, %L)', v_jobmon_schema, v_job_id, 'logdel_maker() failure. Cleaned up source table mimeo objects (queue table, function & trigger) if they existed.') INTO v_step_id;
+            END IF;
+            IF v_jobmon THEN
+                EXECUTE format('SELECT %I.update_step(%L, %L, %L)', v_jobmon_schema, v_step_id, 'CRITICAL', 'See first step for error message.');
+                EXECUTE format('SELECT %I.fail_job(%L)', v_jobmon_schema, v_job_id);
+            END IF;
             RAISE EXCEPTION 'logdel_maker() failure. Cleaned up source table mimeo objects (queue table, function & trigger) if they existed. SQLERRM: %', SQLERRM;
         ELSE
+            IF v_jobmon THEN
+                EXECUTE format('SELECT %I.add_step(%L, %L)', v_jobmon_schema, v_job_id, 'logdel_maker() failure. Unable to clean up source database objects (trigger/queue table) if they were made.') INTO v_step_id;
+            END IF;
+            IF v_jobmon THEN
+                EXECUTE format('SELECT %I.update_step(%L, %L, %L)', v_jobmon_schema, v_step_id, 'CRITICAL', 'See first step for error message.');
+                EXECUTE format('SELECT %I.fail_job(%L)', v_jobmon_schema, v_job_id);
+            END IF;
             RAISE EXCEPTION 'logdel_maker() failure. Unable to clean up source database objects (trigger/queue table) if they were made. SQLERRM: % ', SQLERRM;
         END IF;
+
 END
 $$;
 
