@@ -1,7 +1,7 @@
 /*
  *  Refresh based on DML (Insert, Update, Delete)
  */
-CREATE FUNCTION refresh_dml(p_destination text, p_limit int default NULL, p_repull boolean DEFAULT false, p_jobmon boolean DEFAULT NULL, p_lock_wait int DEFAULT NULL, p_debug boolean DEFAULT false) RETURNS void
+CREATE FUNCTION refresh_dml(p_destination text, p_limit int default NULL, p_repull boolean DEFAULT false, p_jobmon boolean DEFAULT NULL, p_lock_wait int DEFAULT NULL, p_insert_on_fetch boolean DEFAULT NULL, p_debug boolean DEFAULT false) RETURNS void
     LANGUAGE plpgsql SECURITY DEFINER
     AS $$
 DECLARE
@@ -12,6 +12,7 @@ v_cols_n_types          text;
 v_cols                  text;
 v_condition             text;
 v_control               text;
+v_cursor_name           text;
 v_dblink                int;
 v_dblink_name           text;
 v_dblink_schema         text;
@@ -23,13 +24,14 @@ v_exec_status           text;
 v_fetch_sql             text;
 v_field                 text;
 v_filter                text[];
-v_insert_sql            text;
+v_insert_on_fetch       boolean;
 v_job_id                int;
 v_jobmon_schema         text;
 v_job_name              text;
 v_jobmon                boolean;
 v_limit                 int; 
 v_link_exists           boolean;
+v_local_insert_sql      text;
 v_old_search_path       text;
 v_pk_counter            int;
 v_pk_name_csv           text;
@@ -105,6 +107,7 @@ SELECT source_table
     , condition
     , batch_limit 
     , jobmon
+    , insert_on_fetch
 INTO v_source_table
     , v_dest_table
     , v_dblink
@@ -115,6 +118,7 @@ INTO v_source_table
     , v_condition
     , v_limit
     , v_jobmon
+    , v_insert_on_fetch
 FROM refresh_config_dml 
 WHERE dest_table = p_destination; 
 IF NOT FOUND THEN
@@ -123,6 +127,7 @@ END IF;
 
 -- Allow override with parameter
 v_jobmon := COALESCE(p_jobmon, v_jobmon);
+v_insert_on_fetch := COALESCE(p_insert_on_fetch, v_insert_on_fetch);
 
 SELECT schemaname, tablename 
 INTO v_dest_schema_name, v_dest_table_name
@@ -223,6 +228,9 @@ IF v_jobmon THEN
     PERFORM update_step(v_step_id, 'OK','Result was '||v_exec_status);
 END IF;
 
+-- Ensure name is consistent in case it would get truncated by maximium object name length
+v_cursor_name := @extschema@.check_name_length('mimeo_cursor_' || v_src_table_name, p_convert_standard := true);
+
 IF p_repull THEN
     IF v_jobmon THEN
         PERFORM update_step(v_step_id, 'OK','Request to repull ALL data from source. This could take a while...');
@@ -246,7 +254,7 @@ ELSE
     EXECUTE format('CREATE TEMP TABLE refresh_dml_queue (%s, PRIMARY KEY (%s))', v_pk_name_type_csv, v_pk_name_csv);
     -- Copy queue locally for use in removing updated/deleted rows
     v_remote_q_sql := format('SELECT DISTINCT %s FROM %I.%I WHERE processed = true', v_pk_name_csv, v_q_schema_name, v_q_table_name);
-    PERFORM dblink_open(v_dblink_name, 'mimeo_cursor', v_remote_q_sql);
+    PERFORM dblink_open(v_dblink_name, v_cursor_name, v_remote_q_sql);
     IF v_jobmon THEN
         v_step_id := add_step(v_job_id, 'Creating local queue temp table');
     END IF;
@@ -256,7 +264,7 @@ ELSE
                 , v_pk_name_csv
                 , v_pk_name_csv
                 , v_dblink_name
-                , 'mimeo_cursor'
+                , v_cursor_name
                 , '50000'
                 , v_pk_name_type_csv);
         EXECUTE v_fetch_sql;
@@ -268,7 +276,7 @@ ELSE
             PERFORM update_step(v_step_id, 'PENDING', 'Fetching rows in batches: '||v_total||' done so far.');
         END IF;
     END LOOP;
-    PERFORM dblink_close(v_dblink_name, 'mimeo_cursor');
+    PERFORM dblink_close(v_dblink_name, v_cursor_name);
     EXECUTE format('CREATE INDEX ON refresh_dml_queue (%s)', v_pk_name_csv);
     ANALYZE refresh_dml_queue;
     IF v_jobmon THEN
@@ -294,36 +302,61 @@ ELSE
     END IF;
 END IF;
 
--- insert records to local table. Have to do temp table in case destination table is partitioned (returns 0 when inserting to parent)
-PERFORM dblink_open(v_dblink_name, 'mimeo_cursor', v_remote_f_sql);
+-- insert records to local table. Have to do temp table in case destination table is non-natively partitioned (returns 0 when inserting to parent). Also allows for when insert_on_fetch is false to reduce the open cursor time on the source.
 IF v_jobmon THEN
     v_step_id := add_step(v_job_id, 'Inserting new records into local table');
 END IF;
 EXECUTE format('CREATE TEMP TABLE refresh_dml_full (%s)', v_cols_n_types);
 v_rowcount := 0;
 v_total := 0;
+v_local_insert_sql := format('INSERT INTO %I.%I (%s) SELECT %s FROM refresh_dml_full', v_dest_schema_name, v_dest_table_name, v_cols, v_cols);
+PERFORM dblink_open(v_dblink_name, v_cursor_name, v_remote_f_sql);
 LOOP
     v_fetch_sql := format('INSERT INTO refresh_dml_full (%s) SELECT %s FROM dblink_fetch(%L, %L, %s) AS (%s)'
             , v_cols
             , v_cols
             , v_dblink_name
-            , 'mimeo_cursor'
+            , v_cursor_name 
             , '50000'
             , v_cols_n_types);
     EXECUTE v_fetch_sql;
     GET DIAGNOSTICS v_rowcount = ROW_COUNT;
     v_total := v_total + coalesce(v_rowcount, 0);
-    EXECUTE format('INSERT INTO %I.%I (%s) SELECT %s FROM refresh_dml_full', v_dest_schema_name, v_dest_table_name, v_cols, v_cols);
-    EXECUTE 'TRUNCATE refresh_dml_full';
-    EXIT WHEN v_rowcount = 0;
     PERFORM gdb(p_debug,'Fetching rows in batches: '||v_total||' done so far.');
-    IF v_jobmon THEN
+    IF v_jobmon AND v_insert_on_fetch THEN
+        -- Avoid the overhead of jobmon logging each batch step to minimize transaction time on source when insert_on_fetch is false
         PERFORM update_step(v_step_id, 'PENDING', 'Fetching rows in batches: '||v_total||' done so far.');
     END IF;
+
+    IF v_insert_on_fetch THEN
+        EXECUTE v_local_insert_sql;
+        EXECUTE 'TRUNCATE refresh_dml_full';
+    END IF;
+
+    IF v_rowcount = 0 THEN
+        -- Above rowcount variable is saved after temp table inserts. 
+        -- So when temp table has the whole queue, this block of code should be reached
+        PERFORM dblink_close(v_dblink_name, v_cursor_name);
+        IF v_insert_on_fetch = false THEN
+            PERFORM gdb(p_debug,'Inserting into destination table in single batch (insert_on_fetch set to false)');
+            IF v_jobmon THEN
+                PERFORM update_step(v_step_id, 'PENDING', 'Inserting into destination table in single batch (insert_on_fetch set to false)...');
+            END IF;
+            EXECUTE v_local_insert_sql;
+            IF v_jobmon THEN
+                PERFORM update_step(v_step_id, 'OK', 'Inserted into destination table in single batch (insert_on_fetch set to false)');
+            END IF;
+        END IF;
+        EXIT; -- leave insert loop
+    END IF;
+
 END LOOP;
-PERFORM dblink_close(v_dblink_name, 'mimeo_cursor');
 IF v_jobmon THEN
-    PERFORM update_step(v_step_id, 'OK','Number of rows inserted: '||v_total);
+    IF v_insert_on_fetch THEN
+        PERFORM update_step(v_step_id, 'OK','Number of rows inserted: '||v_total);
+    ELSE
+        PERFORM update_step(v_step_id, 'OK','Number of rows inserted: '||v_total||'. Final insert to destination done as single batch (insert_on_fetch set to false)');
+    END IF;
 END IF;
 
 IF p_repull = false AND v_total > (v_limit * .75) THEN
@@ -352,8 +385,8 @@ UPDATE refresh_config_dml SET last_run = CURRENT_TIMESTAMP WHERE dest_table = v_
 IF v_jobmon THEN
     PERFORM update_step(v_step_id, 'OK','Last run was '||CURRENT_TIMESTAMP);
 END IF;
-EXECUTE 'DROP TABLE IF EXISTS refresh_dml_full';
-EXECUTE 'DROP TABLE IF EXISTS refresh_dml_queue';
+DROP TABLE IF EXISTS refresh_dml_full;
+DROP TABLE IF EXISTS refresh_dml_queue;
 
 PERFORM dblink_disconnect(v_dblink_name);
 
