@@ -14,6 +14,7 @@ v_cols                  text;
 v_condition             text;
 v_control               text;
 v_create_sql            text;
+v_cursor_name           text;
 v_dblink                int;
 v_dblink_name           text;
 v_dblink_schema         text;
@@ -239,9 +240,12 @@ ELSE
 
 END IF;
 
+-- Ensure name is consistent in case it would get truncated by maximium object name length
+v_cursor_name := @extschema@.check_name_length('mimeo_cursor_' || v_src_table_name, p_convert_standard := true);
+
 EXECUTE format('CREATE TEMP TABLE mimeo_refresh_inserter_temp (%s)', v_cols_n_types);
 PERFORM gdb(p_debug, 'v_remote_sql: '||COALESCE(v_remote_sql, '<NULL>'));
-PERFORM dblink_open(v_dblink_name, 'mimeo_cursor', v_remote_sql);
+PERFORM dblink_open(v_dblink_name, v_cursor_name, v_remote_sql);
 IF v_jobmon THEN
     v_step_id := add_step(v_job_id, 'Inserting new records into local table');
 END IF;
@@ -252,7 +256,7 @@ LOOP
         , v_cols
         , v_cols
         , v_dblink_name
-        , 'mimeo_cursor'
+        , v_cursor_name 
         , '50000'
         , v_cols_n_types);
     EXECUTE v_fetch_sql;
@@ -269,7 +273,7 @@ LOOP
         PERFORM update_step(v_step_id, 'PENDING', 'Fetching rows in batches: '||v_total||' done so far. Last fetched: '||v_last_fetched);
     END IF;
 END LOOP;
-PERFORM dblink_close(v_dblink_name, 'mimeo_cursor');
+PERFORM dblink_close(v_dblink_name, v_cursor_name);
 IF v_jobmon THEN
     PERFORM update_step(v_step_id, 'OK','Rows fetched: '||v_total);
 END IF;
@@ -392,3 +396,4 @@ EXCEPTION
 END
 $$;
 
+
