@@ -14,6 +14,7 @@ v_cols                  text;
 v_condition             text;
 v_control               text;
 v_create_sql            text;
+v_cursor_name           text;
 v_dblink                int;
 v_dblink_name           text;
 v_dblink_schema         text;
@@ -250,9 +251,12 @@ ELSE
 
 END IF;
 
+-- Ensure name is consistent in case it would get truncated by maximium object name length
+v_cursor_name := @extschema@.check_name_length('mimeo_cursor_' || v_src_table_name, p_convert_standard := true);
+
 EXECUTE 'CREATE TEMP TABLE mimeo_refresh_inserter_temp ('||v_cols_n_types||')';
 PERFORM gdb(p_debug,v_remote_sql);
-PERFORM dblink_open(v_dblink_name, 'mimeo_cursor', v_remote_sql);
+PERFORM dblink_open(v_dblink_name, v_cursor_name, v_remote_sql);
 IF v_jobmon THEN
     v_step_id := add_step(v_job_id, 'Inserting new records into local table');
 END IF;
@@ -263,7 +267,7 @@ LOOP
         , v_cols
         , v_cols
         , v_dblink_name
-        , 'mimeo_cursor'
+        , v_cursor_name
         , '50000'
         , v_cols_n_types);
     EXECUTE v_fetch_sql;
@@ -280,7 +284,7 @@ LOOP
         PERFORM update_step(v_step_id, 'PENDING', 'Fetching rows in batches: '||v_total||' done so far. Last fetched: '||v_last_fetched);
     END IF;
 END LOOP;
-PERFORM dblink_close(v_dblink_name, 'mimeo_cursor');
+PERFORM dblink_close(v_dblink_name, v_cursor_name);
 IF v_jobmon THEN
     PERFORM update_step(v_step_id, 'OK','Rows fetched: '||v_total);
 END IF;
@@ -344,11 +348,13 @@ IF v_batch_limit_reached <> 3 THEN
         v_step_id := add_step(v_job_id, 'Setting next lower boundary');
     END IF;
     EXECUTE format('SELECT max(%I) FROM %I.%I', v_control, v_dest_schema_name, v_dest_table_name) INTO v_last_value;
-    UPDATE refresh_config_inserter_time SET last_value = coalesce(v_last_value, CURRENT_TIMESTAMP), last_run = CURRENT_TIMESTAMP WHERE dest_table = p_destination;  
-    IF v_jobmon THEN
-        PERFORM update_step(v_step_id, 'OK','Lower boundary value is: '|| coalesce(v_last_value, CURRENT_TIMESTAMP));
-        PERFORM gdb(p_debug, 'Lower boundary value is: '||coalesce(v_last_value, CURRENT_TIMESTAMP));
+    IF v_last_value IS NOT NULL THEN
+        UPDATE refresh_config_inserter_time SET last_value = v_last_value, last_run = CURRENT_TIMESTAMP WHERE dest_table = p_destination;  
     END IF;
+    IF v_jobmon THEN
+        PERFORM update_step(v_step_id, 'OK','Lower boundary value is: '|| coalesce(v_last_value::text, 'NULL'));
+    END IF;
+        PERFORM gdb(p_debug, 'Lower boundary value is: '||coalesce(v_last_value::text, 'NULL'));
 END IF;
 
 DROP TABLE IF EXISTS mimeo_refresh_inserter_temp;
@@ -403,3 +409,4 @@ EXCEPTION
 END
 $$;
 
+
