@@ -97,7 +97,7 @@ SELECT refresh_dml('mimeo_source.dml_test_source');
 SELECT diag('Running refresh for: mimeo_dest.dml_test_dest');
 SELECT refresh_dml('mimeo_dest.dml_test_dest');
 SELECT diag('Running refresh for: mimeo_dest.dml_test_dest_multi');
-SELECT refresh_dml('mimeo_dest.dml_test_dest_multi');
+SELECT refresh_dml('mimeo_dest.dml_test_dest_multi', p_insert_on_fetch := false);
 SELECT diag('Running refresh for: mimeo_dest.dml_test_dest_nodata');
 SELECT refresh_dml('mimeo_dest.dml_test_dest_nodata', p_jobmon := false);
 SELECT diag('Running refresh for: mimeo_dest.dml_test_dest_filter');
@@ -115,7 +115,7 @@ SELECT refresh_logdel('mimeo_source.logdel_test_source');
 SELECT diag('Running refresh for: mimeo_dest.logdel_test_dest');
 SELECT refresh_logdel('mimeo_dest.logdel_test_dest');
 SELECT diag('Running refresh for: mimeo_dest.logdel_test_dest_multi');
-SELECT refresh_logdel('mimeo_dest.logdel_test_dest_multi');
+SELECT refresh_logdel('mimeo_dest.logdel_test_dest_multi', p_insert_on_fetch := false);
 SELECT diag('Running refresh for: mimeo_dest.logdel_test_dest_nodata');
 SELECT refresh_logdel('mimeo_dest.logdel_test_dest_nodata', p_jobmon := false);
 SELECT diag('Running refresh for: mimeo_dest.logdel_test_dest_filter');
